(* Konvertieren von DFM-Dateien
   Object ==> Text und zurck
   J. Rathlev, Okt. 2004
   letzte nderung, Dez. 2006
   *)

unit DfmToText;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, HListBox, JrUtils, WinDesktop, ComCtrls, ExtCtrls,
  XPMan;

const
  Vers = ' Vers. 1.1';
  CopRgt = '(c) 2006 J. Rathlev, IEAP, Uni-Kiel';

  DfmExt = 'dfm';
  TxtExt = 'dtx';

type
  THauptForm = class(TForm)
    Label1: TLabel;
    Label2: TLabel;
    InfoBtn: TSpeedButton;
    EndeBtn: TBitBtn;
    btnConvert: TSpeedButton;
    OpenDialog: TOpenDialog;
    Label3: TLabel;
    DestDirEdit: THistoryCombo;
    DestSelectBtn: TSpeedButton;
    SourceDirEdit: THistoryCombo;
    FileEdit: THistoryCombo;
    SourceSelectBtn: TSpeedButton;
    FileBtn: TSpeedButton;
    StatusBar: TStaticText;
    chbSubDir: TCheckBox;
    rgpDirection: TRadioGroup;
    XPManifest1: TXPManifest;
    procedure FormCreate(Sender: TObject);
    procedure EndeBtnClick(Sender: TObject);
    procedure InfoBtnClick(Sender: TObject);
    procedure DestSelectBtnClick(Sender: TObject);
    procedure FileBtnClick(Sender: TObject);
    procedure btnConvertClick(Sender: TObject);
    procedure SourceSelectBtnClick(Sender: TObject);
    procedure rgpDirectionClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private-Deklarationen }
    ArbPfad,
    PrgPfad,PrgName,
    IniName          : string;
  public
    { Public-Deklarationen }
  end;

var
  HauptForm: THauptForm;

implementation

{$R *.DFM}

uses IniFiles, ShlObj, ShellDirDlg;

const
  IniExt = 'ini';

  (* INI-Sektionen *)
  CfGSekt = 'Config';
  SourceSekt = 'Source';
  DestSekt = 'Dest';
  FileSekt = 'Files';

  (* INI-Variablen *)
  iniDir = 'Direction';
  iniSubDir = 'Subdirectories';

{ ------------------------------------------------------------------- }
(* INI-File lesen *)
procedure THauptForm.FormCreate(Sender: TObject);
var
  IniFile  : TIniFile;
begin
  PrgPfad:=ExtractFilePath(Application.ExeName);
  PrgName:=ExtractFileName(DelExt(Application.ExeName));
  ArbPfad:=GetDesktopFolder(CSIDL_PERSONAL);
  IniName:=Erweiter(ArbPfad,PrgName,IniExt);
  IniFile:=TIniFile.Create(IniName);
  with IniFile do begin
    rgpDirection.ItemIndex:=ReadInteger(CfgSekt,iniDir,0);
    chbSubDir.Checked:=ReadBool(CfgSekt,iniSubDir,false);
    Free;
    end;
  SourceDirEdit.LoadFromIni (IniName,SourceSekt);
  SourceDirEdit.ItemIndex:=0;
  DestDirEdit.LoadFromIni (IniName,DestSekt);
  DestDirEdit.ItemIndex:=0;
  FileEdit.LoadFromIni (IniName,FileSekt);
  FileEdit.ItemIndex:=0;
  end;

procedure THauptForm.FormDestroy(Sender: TObject);
var
  IniFile  : TIniFile;
begin
  IniFile:=TIniFile.Create(IniName);
  with IniFile do begin
    WriteInteger(CfgSekt,iniDir,rgpDirection.ItemIndex);
    WriteBool(CfgSekt,iniSubDir,chbSubDir.Checked);
    Free;
    end;
  end;

procedure THauptForm.FormShow(Sender: TObject);
begin
  rgpDirectionClick(Sender);
  end;

{ ------------------------------------------------------------------- }
(* Programmende *)
procedure THauptForm.EndeBtnClick(Sender: TObject);
begin
  Close;
  end;

{ ------------------------------------------------------------------- }
procedure THauptForm.InfoBtnClick(Sender: TObject);
begin
  InfoDialog('','Convert Delphi DFM files'+Vers+#13+CopRgt);
  end;

{ ------------------------------------------------------------------- }
procedure THauptForm.rgpDirectionClick(Sender: TObject);
begin
  if rgpDirection.ItemIndex=0 then FileEdit.Text:='*.'+DfmExt
  else FileEdit.Text:='*.'+txtExt;
  end;

procedure THauptForm.SourceSelectBtnClick(Sender: TObject);
var
  s,t : string;
begin
  s:=SourceDirEdit.Text;
  if length(s)=0 then s:=ArbPfad;
  if rgpDirection.ItemIndex=0 then t:='Directory with binary files'
  else t:='Directory with text files';
  if ShellDirDialog.Execute (t,false,'',s) then begin
    with SourceDirEdit do begin
      Text:=s; AddItem(s);
      end;
    with DestDirEdit do if length(Text)=0 then begin
      Text:=s; AddItem(s);
      end;
    rgpDirectionClick(Sender);
    end;
  end;

procedure THauptForm.DestSelectBtnClick(Sender: TObject);
var
  s,t : string;
begin
  s:=DestDirEdit.Text;
  if length(s)=0 then s:=SourceDirEdit.Text;
  if length(s)=0 then s:=ArbPfad;
  if rgpDirection.ItemIndex=1 then t:='Directory with binary files'
  else t:='Directory with text files';
  if ShellDirDialog.Execute (t,false,SourceDirEdit.Text,s) then begin
    with DestDirEdit do begin
      Text:=s; AddItem(s);
      end;
    end;
  end;

procedure THauptForm.FileBtnClick(Sender: TObject);
var
  i : integer;
  s : string;
begin
  with OpenDialog do begin
    InitialDir:=SourceDirEdit.Text;
    Filename:='';
    if rgpDirection.ItemIndex=0 then begin
      Filter:='DFM files|*.'+DfmExt+'|all|*.*';
      Title:='Select DFM files';
      end
    else begin
      Filter:='Text files|*.'+TxtExt+'|all|*.*';
      Title:='Select DFM files';
      end;
    if Execute then begin
      chbSubDir.Checked:=false;
      s:=ExtractFilePath(Files.Strings[0]);
      with SourceDirEdit do if s<>Text then begin
        Text:=s; AddItem(s);
        end;
      with Files do begin
        s:='';
        for i:=0 to Count-1 do s:=s+ExtractFilename(Strings[i])+';';
        end;
      delete (s,length(s),1);
      with FileEdit do begin
        Text:=s; AddItem(s);
        end;
      end;
    end;
  end;

procedure THauptForm.btnConvertClick(Sender: TObject);
var
  FListe,TextList   : TStringList;
  fcnt,i        : integer;
  s,t,dp,
  fname,oname   : string;
  fStream       : TFileStream;
  mStream       : TMemoryStream;
  frm           : TStreamOriginalFormat;

  procedure ScanDir (Dir,Mask : string);
  var
    DirInfo    : TSearchRec;
    Findresult : integer;
  begin
    if chbSubDir.Checked then begin
      FindResult:=FindFirst (Erweiter(Dir,'*','*'),faAnyFile,DirInfo);
      while FindResult=0 do with DirInfo do begin
        if NotSpecialDir(Name) then begin
          if((Attr and faDirectory)<>0) then ScanDir (Erweiter(Dir,Name,''),Mask);
          end;
        FindResult:=FindNext (DirInfo);
        end;
      FindClose(DirInfo);
      end;
    FindResult:=FindFirst (Erweiter(Dir,Mask,''),faArchive+faReadOnly+faHidden+faSysfile,DirInfo);
    while FindResult=0 do with DirInfo do begin
      if NotSpecialDir(Name) then FListe.Add(Erweiter(Dir,Name,''));
      FindResult:=FindNext (DirInfo);
      end;
    FindClose(DirInfo);
    end;


begin
  t:=FileEdit.Text;
  if length(t)>0 then begin
    Screen.Cursor:=crHourglass;
    with StatusBar do begin
      Font.Color:=clBlack;
      Caption:='';
      end;
    TextList:=TStringList.Create;;
    FListe:=TStringList.Create;
    fcnt:=0;
    repeat
      s:=ReadNxtStr (t,';');
      FListe.Clear;
      ScanDir (SourceDirEdit.Text,s);
      fcnt:=fcnt+FListe.Count;
      with FListe do if Count>0 then for i:=0 to Count-1 do begin
        fName:=Strings[i];
        oName:=ExtractRelativePath(SetDirName(SourceDirEdit.Text),Strings[i]);
        StatusBar.Caption:=oName;
        oName:=SetDirName(DestDirEdit.Text)+oName;
        dp:=ExtractFilePath(oName);
        if not DirectoryExists(dp) then ForceDirectories(dp);
        TextList.Clear;
        mStream := TMemoryStream.Create;
        if rgpDirection.ItemIndex=0 then begin
          oName:=NewExt(oName,TxtExt);
          fStream := TFileStream.Create(fName,fmOpenRead or fmShareDenyNone);
          frm:=sofText;
          try
            ObjectResourceToText(fStream,mStream,frm);
            mStream.Position := 0;
            with TextList do begin
              LoadFromStream(mStream);
              SaveToFile(oName);
              end;
          except
            ErrorDialog (Caption,'Error converting: '#13+fName);
//            CopyFileS (fName,oName);
            end;
          end
        else begin
          oName:=NewExt(oName,DfmExt);
          fStream := TFileStream.Create(oName,fmCreate or fmShareExclusive);
          frm:=sofBinary;
          with TextList do begin
            LoadFromFile(fName);
            SaveToStream(mStream);
            end;
          mStream.Position := 0;
          ObjectTextToResource(mStream,fStream,frm);
          end;
        mStream.Free;
        fStream.Free;
        end;
      until length(t)=0;
    FListe.Free; TextList.Free;
    with StatusBar do begin
      Font.Color:=clBlue;
      Caption:=' '+IntToStr(fcnt)+' Files converted';
      end;
    Screen.Cursor:=crDefault;
    end
  else StatusBar.Caption:=' Nothing to do!';
  end;

end.
