(* Extrahieren der Textstrings aus DFM-Dateien
   Diese mssen zuvor mit "DfmConvert" ins Textformat konvertiert werden
   J. Rathlev, Okt. 2004
   letzte nderung, Dez. 2006
   *)

unit RdFrmStrings;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, HListBox, JrUtils, WinDesktop, ComCtrls,
  ExtCtrls, XPMan;

const
  Vers = ' Vers. 1.1';
  CopRgt = '(c) 2006 J. Rathlev, IEAP, Uni-Kiel';

  StrExt = 'txt';
  FrmExt = 'dtx';

type
  THauptForm = class(TForm)
    Label1: TLabel;
    Label2: TLabel;
    InfoBtn: TSpeedButton;
    EndeBtn: TBitBtn;
    btnExtract: TSpeedButton;
    OpenDialog: TOpenDialog;
    Label3: TLabel;
    StrFileEdit: THistoryCombo;
    FormsDirEdit: THistoryCombo;
    FileEdit: THistoryCombo;
    FormsDirBtn: TSpeedButton;
    FileBtn: TSpeedButton;
    StatusBar: TStaticText;
    chbSubDir: TCheckBox;
    XPManifest1: TXPManifest;
    procedure FormCreate(Sender: TObject);
    procedure EndeBtnClick(Sender: TObject);
    procedure InfoBtnClick(Sender: TObject);
    procedure FileBtnClick(Sender: TObject);
    procedure btnExtractClick(Sender: TObject);
    procedure FormsDirBtnClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private-Deklarationen }
    ArbPfad,
    PrgPfad,PrgName,
    IniName          : string;
  public
    { Public-Deklarationen }
  end;

var
  HauptForm: THauptForm;

implementation

{$R *.DFM}

uses ShlObj, StrUtils, IniFiles, ShellDirDlg;

const
  IniExt = 'ini';

  (* INI-Sektionen *)
  CfGSekt = 'Config';
  SourceSekt = 'Source';
  DestSekt = 'Dest';
  FileSekt = 'Files';

  (* INI-Variablen *)
  iniSubDir = 'Subdirectories';

{ ------------------------------------------------------------------- }
(* INI-File lesen *)
procedure THauptForm.FormCreate(Sender: TObject);
var
  IniFile  : TIniFile;
begin
  PrgPfad:=ExtractFilePath(Application.ExeName);
  PrgName:=ExtractFileName(DelExt(Application.ExeName));
  ArbPfad:=GetDesktopFolder(CSIDL_PERSONAL);
  IniName:=Erweiter(ArbPfad,PrgName,IniExt);
  IniFile:=TIniFile.Create(IniName);
  with IniFile do begin
    chbSubDir.Checked:=ReadBool(CfgSekt,iniSubDir,false);
    Free;
    end;
  FormsDirEdit.LoadFromIni (IniName,SourceSekt);
  FormsDirEdit.ItemIndex:=0;
  StrFileEdit.LoadFromIni (IniName,DestSekt);
  StrFileEdit.ItemIndex:=0;
  FileEdit.LoadFromIni (IniName,FileSekt);
  FileEdit.ItemIndex:=0;
  end;

procedure THauptForm.FormDestroy(Sender: TObject);
var
  IniFile  : TIniFile;
begin
  IniFile:=TIniFile.Create(IniName);
  with IniFile do begin
    WriteBool(CfgSekt,iniSubDir,chbSubDir.Checked);
    Free;
    end;
  end;

{ ------------------------------------------------------------------- }
(* Programmende *)
procedure THauptForm.EndeBtnClick(Sender: TObject);
begin
  Close;
  end;

{ ------------------------------------------------------------------- }
procedure THauptForm.InfoBtnClick(Sender: TObject);
begin
  InfoDialog('','Extract strings from Delphi DFM files'+Vers+#13+CopRgt);
  end;

{ ------------------------------------------------------------------- }
procedure THauptForm.FormsDirBtnClick(Sender: TObject);
var
  s : string;
begin
  s:=FormsDirEdit.Text;
  if length(s)=0 then s:=ArbPfad;
  if ShellDirDialog.Execute ('Directory with Text-Forms',false,'',s) then begin
    with FormsDirEdit do begin
      Text:=s; AddItem(s);
      end;
    with FileEdit do begin
      Text:='*.'+FrmExt; AddItem(Text);
      end;
    end;
  end;

procedure THauptForm.FileBtnClick(Sender: TObject);
var
  i : integer;
  s : string;
begin
  with OpenDialog do begin
    InitialDir:=FormsDirEdit.Text;
    Filename:='';
    Filter:='Form files|*.'+FrmExt+'|all|*.*';
    Title:='Select form files';
    if Execute then begin
      chbSubDir.Checked:=false;
      s:=ExtractFilePath(Files.Strings[0]);
      with FormsDirEdit do if s<>Text then begin
        Text:=s; AddItem(s);
        end;
      with Files do begin
        s:='';
        for i:=0 to Count-1 do s:=s+ExtractFilename(Strings[i])+';';
        end;
      delete (s,length(s),1);
      with FileEdit do begin
        Text:=s; AddItem(s);
        end;
      end;
    end;
  end;

procedure THauptForm.btnExtractClick(Sender: TObject);
var
  FListe        : TStringList;
  lObj,
  s,t,oname     : string;
  fText,oText   : TextFile;
  lCnt,sCnt,
  i,n,k         : integer;

  procedure ScanDir (Dir,Mask : string);
  var
    DirInfo    : TSearchRec;
    Findresult : integer;
  begin
    if chbSubDir.Checked then begin
      FindResult:=FindFirst (Erweiter(Dir,'*','*'),faAnyFile,DirInfo);
      while FindResult=0 do with DirInfo do begin
        if NotSpecialDir(Name) then begin
          if((Attr and faDirectory)<>0) then ScanDir (Erweiter(Dir,Name,''),Mask);
          end;
        FindResult:=FindNext (DirInfo);
        end;
      FindClose(DirInfo);
      end;
    FindResult:=FindFirst (Erweiter(Dir,Mask,''),faArchive+faReadOnly+faHidden+faSysfile,DirInfo);
    while FindResult=0 do with DirInfo do begin
      if NotSpecialDir(Name) then FListe.Add(Erweiter(Dir,Name,''));
      FindResult:=FindNext (DirInfo);
      end;
    FindClose(DirInfo);
    end;

const
  KeyWordCount = 6;
  Keywords : array [1..KeyWordCount] of string =
      ('Object ','.Strings =',' Caption =',' Hint =',' Text =',' Filter =');

  function ScanForKeyword (var s : string) : integer;
  var
    i,n : integer;
  begin
    for i:=1 to KeyWordCount do begin
      Result:=i;
      if AnsiContainsText(s,Keywords[i]) then begin
        s:=Trim(s); n:=Pos(Keywords[i],s);
        Delete(s,1,n+length(Keywords[i])-1);
        s:=Trim(s);
        exit;
        end;
      end;
    Result:=0;
    end;

  function ExtractString(s : string) : string;
  var
    t : string;
    j,k,n : integer;
  begin
    s:=Trim(s); t:='';
    if length(s)>0 then repeat
      if s[1]=#39 then begin
        k:=PosEx(#39,s,2);
        if k=0 then k:=length(s)+1;
        t:=t+copy(s,2,k-2);
        delete(s,1,k);
        end
      else if s[1]='#' then begin
        k:=PosEx(#39,s,2); j:=PosEx('#',s,2);
        if (j>0) and (j<k) then k:=j;
        if k=0 then k:=length(s)+1;
        val(copy(s,2,k-2),n,j);
        if (j=0) and (n>=32) and (n<256) then t:=t+chr(n);
        delete(s,1,k-1);
        end
      else if s[1]=')' then delete (s,1,1)
      else begin
        t:=t+s;
        s:='';
        end;
      until length(s)=0;
    Result:=t;
    end;

  procedure WriteItem (s : string);
  begin
    if length(s)>0 then begin
      writeln(oText,'/:',ZStrint(lCnt,5));
      writeln(oText,'//',s);
      writeln(oText,s);
      writeln(oText);
      inc(sCnt);
      end;
    end;

begin
  t:=FileEdit.Text;
  if length(t)>0 then begin
    Screen.Cursor:=crHourglass;
    with StatusBar do begin
      Font.Color:=clBlack;
      Caption:='';
      end;
    FListe:=TStringList.Create;
    sCnt:=0;
    oName:=SetDirName(FormsDirEdit.Text)+NewExt(StrFileEdit.Text,StrExt);
    AssignFile(oText,oName); rewrite(oText);
    repeat
      s:=ReadNxtStr (t,';');
      FListe.Clear;
      ScanDir (FormsDirEdit.Text,s);
      with FListe do if Count>0 then for i:=0 to Count-1 do begin
        lObj:=''; lCnt:=0;
        AssignFile(fText,Strings[i]); reset(fText);
        s:=ExtractRelativePath(SetDirName(FormsDirEdit.Text),Strings[i]);
        writeln(oText,'//',FormsDirEdit.Text);
        writeln(oText,'/!',s);
        writeln(oText,'//',StringOfChar('-',length(s)));
        writeln(oText);
        while not Eof(fText) do begin
          readln(fText,s); inc(lCnt);
          n:=ScanForKeyword(s);
          if n>0 then case n of
          1 : begin// object
              k:=Pos(':',s);
              lObj:=copy(s,1,k-1);
              end;
          2 : begin // Strings
              while (length(s)>0) and (s[length(s)]<>')') do begin
                readln(fText,s); inc(lCnt);
                WriteItem(ExtractString(s));
                end;
              end;
            else begin  // andere
              s:=ExtractString(s);
              if (n<>3) or not ((AnsiCompareText(s,lObj)=0) or (s='-')) then WriteItem(s);              end;
            end;
          end;
        CloseFile(fText);
        writeln(oText,'//',StringOfChar('=',60));
        end;
      until length(t)=0;
    CloseFile(oText);
    FListe.Free;
    with StatusBar do begin
      Font.Color:=clBlue;
      Caption:=' '+IntToStr(sCnt)+' Strings extracted';
      end;
    Screen.Cursor:=crDefault;
    end
  else StatusBar.Caption:=' Nothing to do!';
  end;

end.
