(* Einfgen der Textstrings in DFM-Dateien
   Diese knnen anschlieend mit "DfmConvert" ins Binrformat konvertiert werden
   J. Rathlev, Okt. 2004
   letzte nderung, Dez. 2006
   *)

unit WrDrcStrings;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, HListBox, JrUtils, WinDesktop, ComCtrls,
  ExtCtrls, XPMan;

const
  Vers = ' Vers. 1.1';
  CopRgt = '(c) 2006 J. Rathlev, IEAP, Uni-Kiel';

  StrExt = 'txt';
  DrcExt = 'rc';
  ErrExt = 'err';

type
  THauptForm = class(TForm)
    Label1: TLabel;
    InfoBtn: TSpeedButton;
    EndeBtn: TBitBtn;
    btnInsert: TSpeedButton;
    OpenDialog: TOpenDialog;
    Label3: TLabel;
    StrFileEdit: THistoryCombo;
    FileEdit: THistoryCombo;
    StatusBar: TStaticText;
    FileBtn: TSpeedButton;
    XPManifest1: TXPManifest;
    procedure FormCreate(Sender: TObject);
    procedure EndeBtnClick(Sender: TObject);
    procedure InfoBtnClick(Sender: TObject);
    procedure btnInsertClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FileBtnClick(Sender: TObject);
  private
    { Private-Deklarationen }
    ArbPfad,
    PrgPfad,PrgName,
    IniName          : string;
  public
    { Public-Deklarationen }
  end;

var
  HauptForm: THauptForm;

implementation

{$R *.DFM}

uses ShlObj, StrUtils;

const
  IniExt = 'ini';

  (* INI-Sektionen *)
  CfGSekt = 'Config';
  SourceSekt = 'Source';
  DestSekt = 'Dest';
  FileSekt = 'Files';

  (* INI-Variablen *)

{ ------------------------------------------------------------------- }
(* INI-File lesen *)
procedure THauptForm.FormCreate(Sender: TObject);
begin
  PrgPfad:=ExtractFilePath(Application.ExeName);
  PrgName:=ExtractFileName(DelExt(Application.ExeName));
  ArbPfad:=GetDesktopFolder(CSIDL_PERSONAL);
  IniName:=Erweiter(ArbPfad,PrgName,IniExt);
  FileEdit.LoadFromIni (IniName,DestSekt);
  FileEdit.ItemIndex:=0;
  StrFileEdit.LoadFromIni (IniName,SourceSekt);
  StrFileEdit.ItemIndex:=0;
  end;

procedure THauptForm.FormDestroy(Sender: TObject);
begin
  end;

{ ------------------------------------------------------------------- }
(* Programmende *)
procedure THauptForm.EndeBtnClick(Sender: TObject);
begin
  Close;
  end;

{ ------------------------------------------------------------------- }
procedure THauptForm.InfoBtnClick(Sender: TObject);
begin
  InfoDialog('','Insert strings to Delphi RC file'+Vers+#13+CopRgt);
  end;

{ ------------------------------------------------------------------- }
procedure THauptForm.FileBtnClick(Sender: TObject);
begin
  with OpenDialog do begin
    if length(StrFileEdit.Text)>0 then InitialDir:=ExtractFilePath(StrFileEdit.Text)
    else InitialDir:=ArbPfad;
    Filename:=ExtractFileName(StrFileEdit.Text);
    Filter:='Text file|*.'+StrExt+'|all|*.*';
    Title:='Select resource text file';
    if Execute then with StrFileEdit do begin
      Text:=Filename; AddItem(Filename);
      with FileEdit do begin
        Text:=NewExt(Filename,DrcExt); AddItem(Text);
        end;
      end;
    end;
  end;

procedure THauptForm.btnInsertClick(Sender: TObject);
var
  FDrc         : TStringList;
  s,t,ename     : string;
  fText,fErr    : TextFile;
  lCnt,sCnt,eCnt,
  i,n,k         : integer;

  function ReadInt (var s    : String;
                    Del      : char;
                    var Wert : integer) : boolean;
  var
    n    : integer;
    i,ic : integer;
  begin
    s:=RemSp(s); i:=pos(Del,s);
    if i=0 then i:=succ(length(s));
    val(copy(s,1,pred(i)),n,ic);
    if ic=0 then begin
      Result:=true;
      Wert:=n; end
    else begin
      Wert:=0; Result:=false;
      end;
    delete(s,1,i);
    end;

  function ReplaceString(var s : string; t : string) : boolean;
  var
    k1,k2 : integer;
  begin
    k1:=Pos('"',s);
    if k1>0 then begin
      repeat
        k2:=PosEx('"',s,k2+1);
        until (k2=0) or (s[k2-1]<>'\');
      s:=copy(s,1,k1-1)+'"'+t+'"';
      Result:=true;
      inc(sCnt);
      end
    else Result:=false;
    end;
    
  procedure WriteError (s,t : string);
  begin
    inc(eCnt);
    Writeln (fErr,s+' error in line: ',StrInt(lCnt,0),' => ',t);
    end;

begin
  Screen.Cursor:=crHourglass;
  with StatusBar do begin
    Font.Color:=clBlack;
    Caption:='';
    end;
  FDrc:=TStringList.Create;
  sCnt:=0; lCnt:=0; eCnt:=0;
  if FileExists(StrFileEdit.Text) then begin
    AssignFile(fText,StrFileEdit.Text); reset(fText);
    eName:=NewExt(StrFileEdit.Text,ErrExt);
    AssignFile(fErr,eName); rewrite (fErr);
    FDrc.LoadFromFile(FileEdit.Text);
    while not Eof(fText) do begin
      readln(fText,s); inc(lCnt);
      t:=copy(s,1,2); delete(s,1,2);
      if t='/:' then begin  // string
        if ReadInt (s,' ',n) then begin
          repeat
            readln(fText,s); inc(lCnt);    // read string
            until copy(s,1,2)<>'//';
          if (n<=0) or (n>FDrc.Count) then begin
            WriteError('Line',s);
            writeln (fErr,'  Line number: ',StrInt(n,0),' not found');
            end
          else with FDrc do begin
            t:=Strings[n-1];
            if ReplaceString(t,s) then Strings[n-1]:=t
            else begin // error
              WriteError('Replace',s);
              writeln (fErr,'  String in line: ',StrInt(n,0),' => ',Strings[n-1]);
              end;
            end;
          end
        else WriteError ('Number',s);  // error
        end;
      end;
    CloseFile(fText); CloseFile(fErr);
    with FDrc do begin
      SaveToFile(FileEdit.Text);
      Free;
      end;
    end
  else ErrorDialog(Caption,'File: '+StrFileEdit.Text+#13'not found!');
  with StatusBar do begin
    Font.Color:=clBlue;
    Caption:=' '+IntToStr(sCnt)+' Strings inserted';
    if eCnt=0 then begin
      Caption:=Caption+' - no errors';
      DeleteFile (eName);
      end
    else Caption:=Caption+' - '+IntToStr(eCnt)+' errors';
    end;
  Screen.Cursor:=crDefault;
  end;

end.
