(* Einfgen der Textstrings in DFM-Dateien
   Diese knnen anschlieend mit "DfmConvert" ins Binrformat konvertiert werden
   J. Rathlev, Okt. 2004
   letzte nderung, Dez. 2006
   *)

unit WrFrmStrings;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, HListBox, JrUtils, WinDesktop, ComCtrls,
  ExtCtrls, XPMan;

const
  Vers = ' Vers. 1.1';
  CopRgt = '(c) 2006 J. Rathlev, IEAP, Uni-Kiel';

  StrExt = 'txt';
  ErrExt = 'err';

type
  THauptForm = class(TForm)
    Label1: TLabel;
    InfoBtn: TSpeedButton;
    EndeBtn: TBitBtn;
    btnInsert: TSpeedButton;
    OpenDialog: TOpenDialog;
    Label3: TLabel;
    StrFileEdit: THistoryCombo;
    FormsDirEdit: THistoryCombo;
    FormDirBtn: TSpeedButton;
    StatusBar: TStaticText;
    FileBtn: TSpeedButton;
    XPManifest1: TXPManifest;
    procedure FormCreate(Sender: TObject);
    procedure EndeBtnClick(Sender: TObject);
    procedure InfoBtnClick(Sender: TObject);
    procedure btnInsertClick(Sender: TObject);
    procedure FormDirBtnClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FileBtnClick(Sender: TObject);
  private
    { Private-Deklarationen }
    ArbPfad,
    PrgPfad,PrgName,
    IniName          : string;
  public
    { Public-Deklarationen }
  end;

var
  HauptForm: THauptForm;

implementation

{$R *.DFM}

uses ShlObj, StrUtils, ShellDirDlg;

const
  IniExt = 'ini';

  (* INI-Sektionen *)
  CfGSekt = 'Config';
  SourceSekt = 'Source';
  DestSekt = 'Dest';
  FileSekt = 'Files';

  (* INI-Variablen *)

{ ------------------------------------------------------------------- }
(* INI-File lesen *)
procedure THauptForm.FormCreate(Sender: TObject);
begin
  PrgPfad:=ExtractFilePath(Application.ExeName);
  PrgName:=ExtractFileName(DelExt(Application.ExeName));
  ArbPfad:=GetDesktopFolder(CSIDL_PERSONAL);
  IniName:=Erweiter(ArbPfad,PrgName,IniExt);
  FormsDirEdit.LoadFromIni (IniName,SourceSekt);
  FormsDirEdit.ItemIndex:=0;
  StrFileEdit.LoadFromIni (IniName,DestSekt);
  StrFileEdit.ItemIndex:=0;
  end;

procedure THauptForm.FormDestroy(Sender: TObject);
begin
  end;

{ ------------------------------------------------------------------- }
(* Programmende *)
procedure THauptForm.EndeBtnClick(Sender: TObject);
begin
  Close;
  end;

{ ------------------------------------------------------------------- }
procedure THauptForm.InfoBtnClick(Sender: TObject);
begin
  InfoDialog('','Insert strings to Delphi DFM files'+Vers+#13+CopRgt);
  Application.MessageBox('Insert strings to Delphi DFM files - Vers. '+Vers+#13+
                         '(c) J. Rathlev - IEAP-Kiel (2004)'#13+
                         'D-24098 Kiel',
                         'Info',mb_OK+mb_IconInformation);
  end;

{ ------------------------------------------------------------------- }
procedure THauptForm.FileBtnClick(Sender: TObject);
begin
  with OpenDialog do begin
    if length(StrFileEdit.Text)>0 then InitialDir:=ExtractFilePath(StrFileEdit.Text)
    else InitialDir:=ArbPfad;
    Filename:=ExtractFileName(StrFileEdit.Text);
    Filter:='String files|*.'+StrExt+'|all|*.*';
    Title:='Select string file';
    if Execute then with StrFileEdit do begin
      Text:=Filename; AddItem(Filename);
      end;
    end;
  end;

procedure THauptForm.FormDirBtnClick(Sender: TObject);
var
  s,t : string;
begin
  s:=FormsDirEdit.Text;
  t:=ExtractFilePath(StrFileEdit.Text);
  if length(s)=0 then s:=ArbPfad;
  if ShellDirDialog.Execute ('Directory for DFM-files',false,t,s) then begin
    with FormsDirEdit do begin
      Text:=s; AddItem(s);
      end;
    end;
  end;

procedure THauptForm.btnInsertClick(Sender: TObject);
var
  FForm         : TStringList;
  FrmName,
  s,t,eName     : string;
  fText,fErr    : TextFile;
  lCnt,sCnt,eCnt,
  n             : integer;

  function ReadInt (var s    : String;
                    Del      : char;
                    var Wert : integer) : boolean;
  var
    n    : integer;
    i,ic : integer;
  begin
    s:=RemSp(s); i:=pos(Del,s);
    if i=0 then i:=succ(length(s));
    val(copy(s,1,pred(i)),n,ic);
    if ic=0 then begin
      Result:=true;
      Wert:=n; end
    else begin
      Wert:=0; Result:=false;
      end;
    delete(s,1,i);
    end;

  function ConvertUml (s : string) : string;
  var
    t : string;
    i : integer;
  begin
    t:=#39;
    if length(s)>0 then begin
      for i:=1 to length(s) do begin
        if s[i]=#39 then t:=t+#39+#39
        else if (s[i]<#32) or (s[i]>#127) then begin
          if t[length(t)]=#39 then delete (t,length(t),1)
          else t:=t+#39;
          t:=t+'#'+StrInt(ord(s[i]),3)+#39;
          end
        else t:=t+s[i];
        end;
      t:=t+#39;
      i:=length(t);
      if t[i-1]=#39 then delete(t,i-1,2);
      end
    else t:=t+#39;
    Result:=t;
    end;

  function ReplaceString(var s : string; t : string) : boolean;
  var
    k1,k2 : integer;
    par   : boolean;
  begin
    k1:=Pos(#39,s); k2:=Pos('#',s);
    if (k2>0) and (k2<k1) then k1:=k2;
    if k1>0 then begin
      par:=s[length(s)]=')';
      s:=copy(s,1,k1-1)+ConvertUml(t);
      if par then s:=s+')';
      Result:=true;
      inc(sCnt);
      end
    else Result:=false;
    end;
    
  procedure WriteError (s,t : string);
  begin
    inc(eCnt);
    Writeln (fErr,s+' error line: ',StrInt(lCnt,0),' => ',t);
    end;

begin
  Screen.Cursor:=crHourglass;
  with StatusBar do begin
    Font.Color:=clBlack;
    Caption:='';
    end;
  FForm:=TStringList.Create;
  sCnt:=0; lCnt:=0; eCnt:=0; FrmName:='';
  if FileExists(StrFileEdit.Text) then begin
    AssignFile(fText,StrFileEdit.Text); reset(fText);
    eName:=NewExt(StrFileEdit.Text,ErrExt);
    AssignFile(fErr,eName); rewrite (fErr);
    while not Eof(fText) do begin
      readln(fText,s); inc(lCnt);
      t:=copy(s,1,2); delete(s,1,2);
      if t='/!' then begin  // filename
        with FForm do begin
          if (length(FrmName)>0) and (Count>0) then SaveToFile(FrmName);
          Clear;
          FrmName:=SetDirName(FormsDirEdit.Text)+s;
          LoadFromFile(FrmName);
          end;
        end;
      if t='/:' then begin  // string
        if ReadInt (s,' ',n) then begin
          repeat
            readln(fText,s); inc(lCnt);    // read string
            until copy(s,1,2)<>'//';
          if (n<=0) or (n>FForm.Count) then begin
            WriteError('Line',s);
            writeln (fErr,'  File: ',FrmName);
            writeln (fErr,'  Line: ',StrInt(n,0),' not found');
            end
          else with FForm do begin
            t:=Strings[n-1];
            if ReplaceString(t,s) then Strings[n-1]:=t
            else begin // error
              WriteError('Replace',s);
              writeln (fErr,'  File: ',FrmName);
              writeln (fErr,'  Line: ',StrInt(n,0),' => ',Strings[n-1]);
              end;
            end;
          end
        else WriteError ('Number',s);  // error
        end;
      end;
    CloseFile(fText); CloseFile(fErr);
    end
  else ErrorDialog(Caption,'File: '+StrFileEdit.Text+#13'not found!');
  FForm.Free;
  with StatusBar do begin
    Font.Color:=clBlue;
    Caption:=' '+IntToStr(sCnt)+' Strings inserted';
    if eCnt=0 then begin
      Caption:=Caption+' - no errors';
      DeleteFile (eName);
      end
    else Caption:=Caption+' - '+IntToStr(eCnt)+' errors';
    end;
  Screen.Cursor:=crDefault;
  end;

end.
